# Contributing to TAM
In order to start contributing to TAM, there are some steps which need to be taken. These are as follows:

1. Get a fork of the TAM repository setup. ([Getting Access](#getting-access))
2. Get the project running on your machine. ([Project Setup](#project-setup))
3. Cover the process of submitting your changes! ([Submitting your changes](#submitting-your-changes))


## Getting access ##
To get started developing for TAM, you will need to request access. 
We do this to prevent TAM development from becoming unmanageable and to keep track of bug bounties effectively.

To request access, you will need to email eip-ewi@tudelft.nl.
In this email, please detail why you want to help out with TAM and what you want to contribute specifically.
Please also disclose your `gitlab.ewi.tudelft.nl` username.
When you get granted access, you will be given your own fork of the project in which you can edit your code.

After you get this access, you can continue to the next steps.


## Project setup ##

In order to have the project setup properly for development a few steps need to be taken.
These steps assume you have already cloned the project (`git clone git@gitlab.ewi.tudelft.nl:eip/labrador/tam.git`)

1. First, the config file should be moved from the template into the actual properties file by executing
   `cp src/main/resources/application.template.yaml src/main/resources/application.yaml`
   from the root directory of the project.
2. Install [IntelliJ IDEA Professional](https://www.jetbrains.com/idea/) ([free for students](https://www.jetbrains.com/student/))
3. For IntelliJ to run the project make sure you are running jdk 17.
4. Start the project by right-clicking `TAMApplication` and clicking 'Run' 
5. Ensure you have [Labracore](https://gitlab.ewi.tudelft.nl/eip/labrador/labracore) running
6. Visit http://localhost:8085/

### Test Data/Users:
* Username: `csestudent1`, Password: `csestudent1`
* Username: `cseteacher1`, Password: `cseteacher1`
* Username: `admin1`, Password: `admin1`
* A list of all users and other things set up in the development environment can be found in the `DevDatabaseLoader` class.


## Submitting your changes ##
After building your awesome new feature time has come to submit a merge request.
The merge request template should already contain a checklist for you to go through, but we also list some steps here.

1. Make sure your code is formatted properly.
   You can check this by using two different gradle tasks: `./gradlew spotlessCheck` and `./gradlew spotlessApply`
2. Ensure your commit message has a title explaining what the change is and a body explaining why you made this change.
3. Ensure your code is tested.
4. Ensure CI passes.
5. Ensure the significant changes you made are listed in the [changelog](./CHANGELOG.md).
   1. Log changes in the [Unreleased] tag
6. Add any necessary Liquibase migrations (instructions for doing this can be found below)


## Liquibase Migrations ##

### Instructions to get Liquibase working: ###
1. Install Liquibase (just from their website)
2. Copy the `liquibase.template.properties` file in the root directory
3. Paste the copy into the root directory and rename it to `liquibase.properties`
4. Check the `build.gradle.kts` file and find the `h2Version` variable. In the `classpath` variable, replace `VERSION` with the version as it appears in the variable.
5. Check the following path in your file explorer: `$HOME\.gradle\caches\modules-2\files-2.1\com.h2database\h2\VERSION` and find the folder containing a `.jar` file. Replace `HASH` in the `classpath` variable with the name of this folder.
   1. If you can't find `modules-2` in the `.gradle/caches` folder, then check this link: https://docs.gradle.org/current/userguide/directory_layout.html and replace `modules-2` with the folder labeled `Shared caches (e.g. for artifacts of dependencies)`.
6. On a new line add the variable `changeSetAuthor` and set it equal to your GitLab username (usually NetID)

### Instructions to create migrations using Liquibase: ###
1. Switch to the `dev` branch
2. Make sure the `application.yaml` file has `url: jdbc:h2:file:./devdb1;DATABASE_TO_UPPER=false` in the `datasource` section.
3. Run TAM normally, there should now be some `.db` files in the root directory
4. Stop TAM
5. Switch back to your branch
6. Make sure the `application.yaml` file has `url: jdbc:h2:mem:devdb` in the `datasource` section.
7. Run TAM
8. Now run this command: `liquibase --changelogFile=new.yaml diff-changelog`, there should now be a file called `new.yaml` in the root directory
9. If you encounter errors during this step, check the `liquibase.properties` file for instructions
10. Now compare the `new.yaml` file to `migrations.yaml` and `table-setup.yaml` and move any changeSets WITH `CREATE_TABLE` into `table-setup.yaml` and all others into `migrations.yaml`
11. Make sure to delete the `new.yaml` file before preforming more migrations


## Production
TAM runs on tam.ewi.tudelft.nl.
This server is managed by the developers of the CSE Teaching Team (eip-ewi@tudelft.nl).