/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.controller;

import java.time.LocalDateTime;
import java.time.LocalTime;

import javax.validation.Valid;

import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.stereotype.Controller;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;

import lombok.AllArgsConstructor;
import nl.tudelft.labracore.api.dto.EditionCreateDTO;
import nl.tudelft.labracore.lib.security.user.AuthenticatedPerson;
import nl.tudelft.labracore.lib.security.user.Person;
import nl.tudelft.tam.dto.create.TamEditionCreateDTO;
import nl.tudelft.tam.service.EditionService;
import nl.tudelft.tam.service.RoleService;

@Controller
@AllArgsConstructor
@RequestMapping("edition")
public class EditionController {

	private EditionService editionService;
	private RoleService roleService;

	/**
	 * Creates a new edition, adds students if selected, and adds creator as teacher.
	 *
	 * @param  person       The creator / currently authenticated person
	 * @param  tamCreateDto The new edition data
	 * @return              The manage job offers page
	 */
	@Validated
	@PostMapping("create")
	@PreAuthorize("@authorisationService.isManagerOfCourse(#tamCreateDto.getCourse().getId())")
	public String createEdition(@AuthenticatedPerson Person person,
			@Valid @ModelAttribute("tamEditionCreateDTO") TamEditionCreateDTO tamCreateDto) {
		// Create the edition
		EditionCreateDTO editionCreateDto = new EditionCreateDTO()
				.name(tamCreateDto.getName())
				.course(tamCreateDto.getCourse())
				.cohort(tamCreateDto.getCohort())
				.enrollability(EditionCreateDTO.EnrollabilityEnum.OPEN)
				.startDate(LocalDateTime.of(tamCreateDto.getStartDate(), LocalTime.of(0, 0)))
				.endDate(LocalDateTime.of(tamCreateDto.getEndDate(), LocalTime.of(23, 59)));

		Long newEditionId = editionService.addEdition(editionCreateDto);

		// Set the creating person as a teacher for the course (ie. manager)
		roleService.changeToTeacher(person.getId(), newEditionId);

		return "redirect:/job-offer/manage";
	}

}
