/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;

import nl.tudelft.tam.security.AuthorisationService;

@RequestMapping
@Controller
public class HomeController {

	@Autowired
	AuthorisationService authorisationService;

	/**
	 * Returns the home page.
	 *
	 * @return the link to the homepage
	 */
	@GetMapping
	public String home() {
		return authorisationService.isAuthenticated() ? "redirect:/job-offer" : "index";
	}

	/**
	 * Returns the error page.
	 *
	 * @return the link to the error page
	 */
	@GetMapping("/error")
	public String error() {
		return "error";
	}

	/**
	 * Returns the privacy statement page.
	 *
	 * @return The privacy page
	 */
	@GetMapping("/privacy")
	public String getPrivacyStatement() {
		return "privacy";
	}

}
