/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.dto.create;

import javax.validation.constraints.NotNull;

import lombok.*;
import nl.tudelft.librador.SpringContext;
import nl.tudelft.librador.dto.create.Create;
import nl.tudelft.tam.enums.Status;
import nl.tudelft.tam.model.Declaration;
import nl.tudelft.tam.repository.ExtraWorkRepository;
import nl.tudelft.tam.repository.ProfileRepository;

@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@EqualsAndHashCode(callSuper = false)
public class DeclarationCreateDTO extends Create<Declaration> {

	@NotNull
	private Long personId;

	@NotNull
	private Long workId;

	@NotNull
	private Integer amount; // amount of quarters, e.g. 1h = 4

	@NotNull
	private String reason;

	private Status status;

	@Override
	public Class<Declaration> clazz() {
		return Declaration.class;
	}

	@Override
	public Declaration apply() {
		validateOrThrow();
		return Declaration.builder()
				.person(SpringContext.getBean(ProfileRepository.class).findByIdOrThrow(personId))
				.extraWork(SpringContext.getBean(ExtraWorkRepository.class).findByIdOrThrow(workId))
				.amount(amount)
				.reason(reason)
				.status(status)
				.build();
	}

}
