/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.dto.create;

import java.time.LocalDate;

import javax.validation.constraints.NotNull;
import javax.validation.constraints.Positive;

import org.apache.commons.lang3.StringUtils;
import org.springframework.format.annotation.DateTimeFormat;

import lombok.*;
import nl.tudelft.librador.dto.create.Create;
import nl.tudelft.tam.enums.AcademicLevel;
import nl.tudelft.tam.enums.AcademicPeriod;
import nl.tudelft.tam.model.JobOffer;

@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@EqualsAndHashCode(callSuper = false)
public class JobOfferCreateDTO extends Create<JobOffer> {

	@NotNull
	private Long editionId;

	private String name;

	private String contractName;

	@DateTimeFormat(pattern = "yyyy-MM-dd")
	private LocalDate contractStartDate;

	@DateTimeFormat(pattern = "yyyy-MM-dd")
	private LocalDate contractEndDate;

	private String baanCode;

	@Positive
	private Integer maxHours;

	@DateTimeFormat(pattern = "yyyy-MM-dd")
	private LocalDate deadline;

	private String description;

	private String hiringMessage;

	private String rejectMessage;

	@Builder.Default
	private Boolean requireApplicationText = false;

	private AcademicLevel level;

	@Builder.Default
	private Boolean hidden = false;

	@Override
	public Class<JobOffer> clazz() {
		return JobOffer.class;
	}

	@Override
	public JobOffer apply() {
		validateOrThrow();
		JobOffer data = new JobOffer();
		data.setEditionId(editionId);
		if (!StringUtils.isBlank(name))
			data.setName(name);
		if (!StringUtils.isBlank(contractName))
			data.setContractName(contractName);
		if (contractStartDate != null)
			data.setContractStartDate(contractStartDate);
		if (contractEndDate != null)
			data.setContractEndDate(contractEndDate);
		if (contractStartDate != null && contractEndDate != null)
			data.setPeriod(
					AcademicPeriod.getAcademicPeriodFromStartAndEndDates(contractStartDate, contractEndDate));
		if (!StringUtils.isBlank(baanCode))
			data.setBaanCode(baanCode);
		if (maxHours > 0)
			data.setMaxHours(maxHours);
		if (deadline != null)
			data.setDeadline(deadline);
		if (!StringUtils.isBlank(description))
			data.setDescription(description);
		if (!StringUtils.isBlank(hiringMessage))
			data.setHiringMessage(hiringMessage);
		if (!StringUtils.isBlank(rejectMessage))
			data.setRejectMessage(rejectMessage);
		if (level != null)
			data.setLevel(level);
		if (hidden != null)
			data.setHidden(hidden);
		postApply(data);
		return data;
	}
}
