/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.dto.create;

import java.time.LocalDate;

import javax.validation.constraints.NotNull;

import org.springframework.format.annotation.DateTimeFormat;

import lombok.*;
import nl.tudelft.librador.SpringContext;
import nl.tudelft.librador.dto.create.Create;
import nl.tudelft.tam.dto.id.ProfileIdDTO;
import nl.tudelft.tam.model.Profile;
import nl.tudelft.tam.model.TrainingApproval;
import nl.tudelft.tam.model.TrainingType;
import nl.tudelft.tam.repository.ProfileRepository;

@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@EqualsAndHashCode(callSuper = false)
public class TrainingApprovalCreateDTO extends Create<TrainingApproval> {

	@NotNull
	private ProfileIdDTO person;

	@NotNull
	private ProfileIdDTO approvedBy;

	@NotNull
	@DateTimeFormat(pattern = "yyyy-MM-dd")
	private LocalDate datePassed;

	@NotNull
	private TrainingType type;

	@Override
	public Class<TrainingApproval> clazz() {
		return TrainingApproval.class;
	}

	@Override
	public void validate() {
		ProfileRepository profileRepository = SpringContext.getBean(ProfileRepository.class);
		if (!profileRepository.existsById(person.getId())) {
			profileRepository.saveAndFlush(Profile.builder().id(person.getId()).build());
		}
	}

}
