/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.model;

import java.time.LocalDate;

import javax.persistence.*;
import javax.validation.constraints.NotNull;

import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import nl.tudelft.tam.enums.PayScale;
import nl.tudelft.tam.enums.Status;

@Data
@Entity
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class RaiseRequest {

	@Id
	@GeneratedValue(strategy = GenerationType.SEQUENCE)
	private Long id;

	@NotNull
	@ManyToOne
	private Profile requestedBy;

	@NotNull
	private PayScale payScale;

	@NotNull
	@Builder.Default
	private Status status = Status.SUBMITTED;

	@ManyToOne
	private Profile approvedBy;

	@NotNull
	@Builder.Default
	@Column(columnDefinition = "DATE")
	private LocalDate createdDate = LocalDate.now();

	@NotNull
	@Builder.Default
	@Column(columnDefinition = "DATE")
	private LocalDate lastModifiedDate = LocalDate.now();

	@PreUpdate
	public void setLastModifiedDate() {
		this.lastModifiedDate = LocalDate.now();
	}

	@PrePersist
	public void setCreatedDate() {
		this.createdDate = LocalDate.now();
	}
}
