/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.repository;

import java.util.List;

import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.rest.webmvc.ResourceNotFoundException;

import nl.tudelft.tam.model.Application;

public interface ApplicationRepository extends JpaRepository<Application, Application.AppId> {

	/**
	 * Finds the application by id or throws a ResourceNotFoundException.
	 *
	 * @param  id the id of the application
	 * @return    the Application object
	 */
	default Application findByIdOrThrow(Application.AppId id) {
		// Spring will actually wrap this in a JpaObjectRetrievalFailureException
		return findById(id)
				.orElseThrow(() -> new ResourceNotFoundException("Application was not found: " + id));
	}

	default List<Application> findAllByPersonId(Long personId) {
		return findAllById_PersonId(personId);
	}

	List<Application> findAllById_PersonId(Long personId);

	default List<Application> findAllByOfferId(Long offerId) {
		return findAllById_JobOfferId(offerId);
	}

	List<Application> findAllById_JobOfferId(Long offerId);

}
