/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.repository;

import java.util.Optional;

import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.rest.webmvc.ResourceNotFoundException;

import nl.tudelft.tam.model.TrainingApproval;
import nl.tudelft.tam.model.TrainingType;

public interface TrainingApprovalRepository extends JpaRepository<TrainingApproval, Long> {

	/**
	 * Finds the trainingApproval by id or throws a ResourceNotFoundException.
	 *
	 * @param  id the id of the trainingApproval
	 * @return    the trainingApproval object
	 */
	default TrainingApproval findByIdOrThrow(Long id) {
		// Spring will actually wrap this in a JpaObjectRetrievalFailureException
		return findById(id)
				.orElseThrow(
						() -> new ResourceNotFoundException("TrainingApproval was not found: " + id));
	}

	/**
	 * Finds training approvals by person id and type (should only ever be empty or 1)
	 *
	 * @param  personId The person for whom to find the approval
	 * @param  type     The type of the approval
	 * @return          The list of approvals matching the criteria (should only every be empty or 1)
	 */
	Optional<TrainingApproval> findByPersonIdAndType(Long personId, TrainingType type);

	/**
	 * Finds training approvals by person id and type id (should only ever be empty or 1)
	 *
	 * @param  personId The person for whom to find the approval
	 * @param  typeId   The id of the type of the approval
	 * @return          The list of approvals matching the criteria (should only every be empty or 1)
	 */
	Optional<TrainingApproval> findByPersonIdAndTypeId(Long personId, Long typeId);

	/**
	 * Delete all training approvals by type
	 *
	 * @param type The type of the approvals to delete
	 */
	void deleteAllByType(TrainingType type);

}
