/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.security;

import static nl.tudelft.labracore.lib.security.user.DefaultRole.*;

import org.springframework.context.annotation.Profile;
import org.springframework.stereotype.Service;

import nl.tudelft.labracore.lib.security.memory.InMemoryUserProvider;

/**
 * Provider for in-memory user information.
 *
 * For more information, consult LabraDoor implementation and documentation at
 * https://gitlab.ewi.tudelft.nl/eip/labrador/labradoor/-/wikis/Introduction.
 */

@Service
@Profile("!production")
public class LoginUserProvider extends InMemoryUserProvider {

	/**
	 * Adds the users created and added inside the constructor to the in-memory database.
	 */
	public LoginUserProvider() {
		super("pass");
		// default password for all users is "pass" (used when no password for user is provided)

		add("Student 1", 11, STUDENT, "student1");
		// username: student1, password: student1

		add("Student 2", 12, STUDENT, "student2");
		// username: student2, password: student2

		add("Teacher 1", 3, TEACHER, "teacher1");
		// username: teacher1, password: teacher1

		add("Teacher 2", 4, TEACHER, "teacher2");
		// username: teacher2, password: teacher2

		add("CSE Teacher 1", 5, TEACHER, "cseteacher1");
		// username: cseteacher1, password: cseteacher1

		add("CSE Teacher 2", 6, TEACHER, "cseteacher2");
		// username: cseteacher2, password: cseteacher2

		add("CSE Teacher 3", 7, TEACHER, "cseteacher3");
		// username: cseteacher3, password: cseteacher3

		add("Admin 1", 8, ADMIN, "admin1");
		// username: admin1, password: admin1

		for (int i = 1; i <= 9; i++) {
			add("CSE Student " + i, 10 + i, STUDENT, "csestudent" + i);
			// username: csestudent{i}, password: csestudent{i}
		}

		add("EE Student 1", 21, STUDENT, "eestudent1");
		// username: eestudent1, password: eestudent1

	}

}
