/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.startup;

import javax.transaction.Transactional;

import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.boot.context.event.ApplicationReadyEvent;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Service;

import lombok.AllArgsConstructor;
import nl.tudelft.tam.repository.JobOfferRepository;

/**
 * A service that runs during startup. This startup procedure goes through all job offers and publishes them
 * in TAM.
 */
@Service
@AllArgsConstructor
@ConditionalOnExpression("#{'${tam.startup.publish-all-job-offers}' == 'true'}")
public class PublishAllJobOffersService {

	private final JobOfferRepository jobOfferRepository;

	@Transactional
	@EventListener(ApplicationReadyEvent.class)
	public void publishAllJobOffers() {
		jobOfferRepository.findAll().forEach(jobOffer -> jobOffer.setHidden(false));
	}
}
