/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.cache;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.Mockito.*;

import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.SpyBean;
import org.springframework.transaction.annotation.Transactional;

import application.test.TestTAMApplication;
import nl.tudelft.labracore.api.EditionControllerApi;
import nl.tudelft.labracore.api.dto.PersonSummaryDTO;
import nl.tudelft.labracore.api.dto.RolePersonDetailsDTO;
import reactor.core.publisher.Flux;

@SpringBootTest(classes = TestTAMApplication.class)
@Transactional
public class EditionRolesCacheManagerTest {

	@SpyBean
	EditionRolesCacheManager cacheManager;

	@Autowired
	private EditionControllerApi eApi;
	final long PERSON_ID = 5001;
	final long PERSON_ID2 = 5002;
	final long EDITION_ID = 7000;

	RolePersonDetailsDTO roleCorrectPersonAndEdition;
	RolePersonDetailsDTO roleInCorrectPersonAndEdition;

	@BeforeEach
	void setup() {
		cacheManager = new EditionRolesCacheManager(eApi);
		roleCorrectPersonAndEdition = new RolePersonDetailsDTO()
				.type(RolePersonDetailsDTO.TypeEnum.STUDENT)
				.person(new PersonSummaryDTO().id(PERSON_ID));
		roleInCorrectPersonAndEdition = new RolePersonDetailsDTO()
				.type(RolePersonDetailsDTO.TypeEnum.TA)
				.person(new PersonSummaryDTO().id(PERSON_ID2));
	}

	@Test
    void testGetRolesInEdition() {
        when(eApi.getEditionParticipants(EDITION_ID))
                .thenReturn(Flux.just(roleCorrectPersonAndEdition, roleInCorrectPersonAndEdition));

        EditionRolesCacheManager.RoleHolder result = cacheManager.getRolesInEdition(PERSON_ID, EDITION_ID);

        verify(eApi, times(1)).getEditionParticipants(EDITION_ID);

        assertEquals(List.of(roleCorrectPersonAndEdition), result.getRoles());
        assertEquals(EDITION_ID, result.getId());
    }

	@Test
    void testGetRolesInEditionEmptyInput() {
        when(eApi.getEditionParticipants(EDITION_ID))
                .thenReturn(Flux.just());

        EditionRolesCacheManager.RoleHolder result = cacheManager.getRolesInEdition(PERSON_ID, EDITION_ID);

        verify(eApi, times(1)).getEditionParticipants(EDITION_ID);

        assertEquals(List.of(), result.getRoles());
        assertEquals(EDITION_ID, result.getId());
    }

	@Test
    void testGetRolesInEditionEmptyResult() {
        when(eApi.getEditionParticipants(EDITION_ID))
                .thenReturn(Flux.just(roleInCorrectPersonAndEdition));

        EditionRolesCacheManager.RoleHolder result = cacheManager.getRolesInEdition(PERSON_ID, EDITION_ID);

        verify(eApi, times(1)).getEditionParticipants(EDITION_ID);

        assertEquals(List.of(), result.getRoles());
        assertEquals(EDITION_ID, result.getId());
    }

}
