/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.controller;

import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.time.LocalDate;
import java.time.LocalDateTime;

import org.junit.jupiter.api.Test;
import org.springframework.transaction.annotation.Transactional;

import nl.tudelft.labracore.api.dto.CohortIdDTO;
import nl.tudelft.labracore.api.dto.CourseIdDTO;
import nl.tudelft.labracore.api.dto.EditionCreateDTO;
import nl.tudelft.labracore.lib.security.user.Person;
import nl.tudelft.tam.dto.create.TamEditionCreateDTO;
import nl.tudelft.tam.service.EditionService;
import nl.tudelft.tam.service.RoleService;

@Transactional
public class EditionControllerTest extends ControllerTest {

	private final EditionController editionController;

	private final EditionService editionService;
	private final RoleService roleService;

	public EditionControllerTest() {
		this.editionService = mock(EditionService.class);
		this.roleService = mock(RoleService.class);

		this.editionController = new EditionController(editionService, roleService);
	}

	@Test
	void createEditionTest() {
		Long personId = randomId();
		Long editionId = randomId();
		Long courseId = randomId();
		Long cohortId = randomId();

		when(editionService.addEdition(any())).thenReturn(editionId);

		editionController.createEdition(Person.builder().id(personId).build(),
				TamEditionCreateDTO.builder()
						.name("Test Edition")
						.course(new CourseIdDTO().id(courseId))
						.cohort(new CohortIdDTO().id(cohortId))
						.startDate(LocalDate.of(2023, 1, 1))
						.endDate(LocalDate.of(2024, 1, 1))
						.build());

		verify(editionService).addEdition(new EditionCreateDTO()
				.name("Test Edition")
				.course(new CourseIdDTO().id(courseId))
				.cohort(new CohortIdDTO().id(cohortId))
				.enrollability(EditionCreateDTO.EnrollabilityEnum.OPEN)
				.startDate(LocalDateTime.of(2023, 1, 1, 0, 0))
				.endDate(LocalDateTime.of(2024, 1, 1, 23, 59)));
		verify(roleService).changeToTeacher(personId, editionId);
	}
}
