/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.controller;

import static org.mockito.Mockito.verify;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.patch;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.util.List;

import javax.transaction.Transactional;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.security.test.context.support.WithUserDetails;
import org.springframework.test.web.servlet.MockMvc;

import application.test.TestTAMApplication;
import nl.tudelft.tam.service.CoordinatorService;

@AutoConfigureMockMvc
@SpringBootTest(classes = TestTAMApplication.class)
@Transactional
public class ProgramControllerTest {

	@Autowired
	private MockMvc mvc;

	@Autowired
	private ProgramController programController;

	@MockBean
	private CoordinatorService coordinatorService;

	@Test
	@WithUserDetails("admin")
	void updateCoordinators() throws Exception {
		mvc.perform(patch("/program/1/coordinators?coordinators=user1\nuser2").with(csrf()))
				.andExpect(status().is3xxRedirection());
		verify(coordinatorService).setCoordinatorsForProgram(1L, List.of("user1", "user2"));
	}

}
