/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.dto;

import static org.assertj.core.api.Assertions.assertThat;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import nl.tudelft.tam.dto.patch.ShirtPatchDTO;
import nl.tudelft.tam.enums.ShirtGender;
import nl.tudelft.tam.enums.ShirtSize;
import nl.tudelft.tam.model.Profile;

class ShirtPatchDTOTest {

	private final ShirtSize defaultSize = ShirtSize.M;
	private final ShirtGender defaultGender = ShirtGender.FEMALE;

	private Profile profile;

	@BeforeEach
	void setup() {
		profile = Profile.builder()
				.id(452L)
				.shirtSize(defaultSize)
				.shirtGender(defaultGender)
				.build();
	}

	@Test
	void updateNone() {
		ShirtPatchDTO patch = new ShirtPatchDTO();
		patch.apply(profile);

		assertThat(profile.getShirtSize()).isEqualTo(defaultSize);
		assertThat(profile.getShirtGender()).isEqualTo(defaultGender);
	}

	@Test
	void updateSize() {
		ShirtPatchDTO patch = new ShirtPatchDTO(ShirtSize.S, null);
		patch.apply(profile);

		assertThat(profile.getShirtSize()).isEqualTo(ShirtSize.S);
		assertThat(profile.getShirtGender()).isEqualTo(defaultGender);
	}

	@Test
	void updateGender() {
		ShirtPatchDTO patch = new ShirtPatchDTO(null, ShirtGender.MALE);
		patch.apply(profile);

		assertThat(profile.getShirtSize()).isEqualTo(defaultSize);
		assertThat(profile.getShirtGender()).isEqualTo(ShirtGender.MALE);
	}

	@Test
	void updateBoth() {
		ShirtPatchDTO patch = new ShirtPatchDTO(ShirtSize.XL, ShirtGender.MALE);
		patch.apply(profile);

		assertThat(profile.getShirtSize()).isEqualTo(ShirtSize.XL);
		assertThat(profile.getShirtGender()).isEqualTo(ShirtGender.MALE);
	}
}
