/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.repository;

import static org.assertj.core.api.Assertions.assertThat;
import static org.junit.jupiter.api.Assertions.*;

import java.util.List;

import javax.persistence.EntityManager;
import javax.transaction.Transactional;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.data.rest.webmvc.ResourceNotFoundException;

import application.test.TestTAMApplication;
import nl.tudelft.tam.model.Coordinator;

@SpringBootTest(classes = TestTAMApplication.class)
public class CoordinatorRepositoryTest {

	@Autowired
	private CoordinatorRepository repository;

	@Autowired
	private EntityManager entityManager;

	@BeforeEach
	void setup() {
		repository.deleteAll();
	}

	@Test
	@Transactional
	void findByIdOrThrow() {
		assertThrows(ResourceNotFoundException.class, () -> repository.findByIdOrThrow(1L));

		Coordinator coordinator = repository
				.saveAndFlush(Coordinator.builder().programId(1L).personId(1L).build());
		entityManager.refresh(coordinator);

		assertThat(repository.findByIdOrThrow(1L)).isEqualTo(coordinator);
	}

	@Test
	@Transactional
	void findAllByPersonIdAndExistsTest() {
		assertThrows(ResourceNotFoundException.class, () -> repository.findByIdOrThrow(1L));
		assertThrows(ResourceNotFoundException.class, () -> repository.findByIdOrThrow(2L));

		assertFalse(repository.existsByPersonId(1L));

		assertFalse(repository.existsByPersonIdAndProgramId(1L, 1L));
		assertFalse(repository.existsByPersonIdAndProgramId(2L, 1L));
		assertFalse(repository.existsByPersonIdAndProgramId(1L, 2L));
		assertFalse(repository.existsByPersonIdAndProgramId(2L, 2L));

		Coordinator coordinator1 = repository.save(Coordinator.builder().programId(1L).personId(1L).build());
		Coordinator coordinator2 = repository.save(Coordinator.builder().programId(2L).personId(2L).build());

		assertTrue(repository.existsByPersonId(1L));

		assertTrue(repository.existsByPersonIdAndProgramId(1L, 1L));
		assertFalse(repository.existsByPersonIdAndProgramId(2L, 1L));
		assertFalse(repository.existsByPersonIdAndProgramId(1L, 2L));
		assertTrue(repository.existsByPersonIdAndProgramId(2L, 2L));

		List<Coordinator> res = repository.findAllByPersonId(1L);
		assertThat(res.size()).isEqualTo(1);
		assertThat(res).contains(coordinator1);
		assertThat(res).doesNotContain(coordinator2);
	}
}
