/*
 * TAM
 * Copyright (C) 2021 - Delft University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package nl.tudelft.tam.service;

import static org.assertj.core.api.Assertions.assertThat;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.ArgumentMatchers.anyLong;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.*;

import java.util.List;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.transaction.annotation.Transactional;

import application.test.TestTAMApplication;
import nl.tudelft.labracore.api.CourseControllerApi;
import nl.tudelft.labracore.api.dto.CourseDetailsDTO;
import nl.tudelft.labracore.api.dto.CourseSummaryDTO;
import nl.tudelft.labracore.api.dto.PersonIdDTO;
import nl.tudelft.tam.cache.CourseCacheManager;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@SpringBootTest(classes = TestTAMApplication.class)
@Transactional
public class CourseServiceTest {
	private final Long COURSE_ID = 110L;
	private final Long PERSON_ID = 550L;

	@Autowired
	public CourseControllerApi courseControllerApi;

	@Autowired
	public CourseService courseService;

	@MockBean
	CourseCacheManager courseCache;

	@Test
	public void getManagingCourses() {
		CourseSummaryDTO course1 = new CourseSummaryDTO().name("Test Course 1");
		CourseSummaryDTO course2 = new CourseSummaryDTO().name("Test Course 2");

		when(courseControllerApi.getAllCoursesByManager(anyLong())).thenReturn(Flux.just(course1, course2));

		assertThat(courseService.getManagingCourses(new PersonIdDTO().id(PERSON_ID)))
				.isEqualTo(List.of(course1, course2));

		verify(courseControllerApi).getAllCoursesByManager(PERSON_ID);
	}

	@Test
	public void getCourseByIdTest() {
		CourseDetailsDTO course = new CourseDetailsDTO().id(1L);
		when(courseControllerApi.getCourseById(anyLong())).thenReturn(Mono.just(course));
		assertEquals(courseService.getCourseById(1L), course);
		verify(courseControllerApi, times(1)).getCourseById(1L);
	}

	@Test
	void getOrThrowTest() {
		CourseDetailsDTO course = new CourseDetailsDTO().id(1L);

		when(courseCache.getOrThrow(anyLong())).thenReturn(course);

		assertThat(courseService.getOrThrow(1L)).isEqualTo(course);

		verify(courseCache).getOrThrow(eq(1L));
	}
}
